/*
 * log functions
 * Copyright (c) 2003 Michel Bardiaux
 *
 * This file is part of FFmpeg.
 *
 * FFmpeg is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * FFmpeg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FFmpeg; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

/**
 * @file
 * logging functions
 */

#include <stdint.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <unistd.h>
#include "av_log.h"
int fft_vsnprintf(char *buf, size_t buflen, const char *__restrict fmt, va_list ap);

static void av_log_default_callback(int level, char *line, unsigned int line_len)
{
}

static void (*av_log_callback)(int, char*, unsigned int) = av_log_default_callback;
int av_log_level = AV_LOG_INFO;

void av_vlog(int level, const char *fmt, va_list vl)
{
    static unsigned int line_len;
    static char line[1024], prev[1024];
    if(level>av_log_level)
        return;

    line_len = vsnprintf(line, sizeof(line)-1, fmt, vl);
    av_log_callback(level, line, line_len);
}

void av_log(int level, const char *fmt, ...)
{
    va_list vl;
    va_start(vl, fmt);
    av_vlog(level, fmt, vl);
    va_end(vl);
}

int av_log_get_level(void)
{
    return av_log_level;
}

void av_log_set_level(int level)
{
    av_log_level = level;
}

void av_log_set_callback(void (*callback)(int, char*, unsigned int))
{
    av_log_callback = callback;
}

