#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <time.h>
#include <sys/stat.h>
#include <sys/mman.h>
//#include "mpegaudio.h"
#include "mp3.h"
#define ID3v2_HEADER_SIZE 10
#define ID3v2_DEFAULT_MAGIC_UINT 0x00334449UL
#define WORD2INT(x) ((x) < -32766.5f ? -32767 : ((x) > 32766.5f ? 32767 : lrintf(x)))
void conv_fltp_to_flt_2ch_sse(float *dst, float *src[2], unsigned int len);
void av_log_set_callback(void (*callback)(int, char*, unsigned int));

typedef struct AVPacket {
    uint8_t *data;
    uint32_t size;
} AVPacket;

static void av_log_callback(int level, char *line, unsigned int line_len)
{
    write(2, line, line_len);
    //write(2, "\n", 1);
}

static unsigned int ff_id3v2_match(const uint8_t *buf)
{
    uint32_t tmp1 = (*(uint32_t*)buf & 0x00FFFFFFUL);
    uint32_t tmp2 = (*(uint32_t*)&buf[6]);
    return ((tmp1 == ID3v2_DEFAULT_MAGIC_UINT) &
            (!!(buf[3] ^ 0xff) & !!(buf[4] ^ 0xff)) &
            ((tmp2 & 0x80808080UL) == 0));
}

static unsigned int id3v2_tag_len(const uint8_t * buf)
{
    uint32_t len = ((buf[6] & 0x7f) << 21) +
                   ((buf[7] & 0x7f) << 14) +
                   ((buf[8] & 0x7f) << 7) +
                    (buf[9] & 0x7f) +
                   ID3v2_HEADER_SIZE;
    if (buf[5] & 0x10)
        len += ID3v2_HEADER_SIZE;
    return len;
}

static const unsigned char static_hdr_portion[8] = {
   0x52, 0x49, 0x46, 0x46, 0xFF, 0xFF, 0xFF, 0x7F,
};

typedef struct _WAVHeader {
    uint32_t wavehdr;
    uint32_t fmt_id;
    uint32_t fmt_len;
    uint16_t wav_id;
    uint16_t channels;
    uint32_t samplerate;
    uint32_t bitrate;
    uint32_t block_align;
    uint32_t pad0;
    uint32_t pad1;
} __attribute__((packed)) WAVHeader;

static void write_wav_header(int fd, uint32_t rate, uint8_t channels)
{
   WAVHeader w;
   write(fd, &static_hdr_portion, 8);

   w.wavehdr = 0x45564157;
   w.fmt_id = 0x20746D66;
   w.fmt_len = 0x10;
   w.wav_id = 1;
   w.channels = channels;
   w.samplerate = rate;
   w.bitrate = rate*channels*2;
   w.block_align = (channels << 1) | 0x00100000;
   w.pad0 = 0x61746164;
   w.pad1 = 0x7fffffff;
   write(fd, &w, sizeof(WAVHeader));
}

/*
 * Audio decoding.
 */
int main(int argc, char **argv)
{
    MPADecodeContext *c;
    struct stat st;
    uint32_t i = 0, out_size;
    int len;
    int in_fd = -1, out_fd = -1;
    unsigned char *inFile, *inFileStart, *inFileEnd;
    uint8_t id3v2_hdr[16];
    float *p[2];
    unsigned int samplerate, header;
    int opt = 1;
    char *devpath = NULL;

    av_log_set_callback(av_log_callback);
    while(opt < argc) {
        if(argv[opt][0] == '-') {
            switch(argv[opt][1]) {
                case 'f':
                case 's':
                  devpath = argv[++opt];
                  break;
                case 'm':
                  break;
                default:
                  break;
            }
        }
        opt++;
    }
    opt--;

    in_fd = open(argv[opt], O_RDONLY, 777);
    if(in_fd < 0) {
        write(1, "File ", 5);
        write(1, argv[1], strlen(argv[1]));
        write(1, " does not exist, exiting.\n", 27);
        return -1;
    }
    fstat(in_fd, &st);

    inFile = mmap(NULL, st.st_size, PROT_READ, MAP_PRIVATE, in_fd, 0);
    inFileStart = inFile;
    inFileEnd = inFile + st.st_size;
    close(in_fd);

    c = mmap(NULL, sizeof(MPADecodeContext), PROT_READ|PROT_WRITE, MAP_PRIVATE|MAP_ANON|MAP_NORESERVE, -1, 0);
    mp3_decode_init(c);
    samplerate = 44100;

	if((out_fd = open(devpath, O_WRONLY|O_CREAT, 0644)) < 0) {
		write(1, "Cannot open sound device (file), exiting.\n", 44);
		return -1;
	}
    write_wav_header(out_fd, samplerate, 2); // ac->channels unless downmixing

    /* First, check for id3v2. Groan. */
    memcpy(id3v2_hdr, inFile, 10);
    if (ff_id3v2_match(id3v2_hdr)) {
        uint64_t id3v2_len = id3v2_tag_len(id3v2_hdr);
        inFile += id3v2_len;
    }

    header = AV_RB32(inFile);
    while (ff_mpa_check_header(header) < 0) {
        inFile += 4;
        header = AV_RB32(inFile);
    }

    /* decode until eof */
    while (inFile < inFileEnd) {
        uint64_t inFileRemaining = (inFileEnd - inFile);
        out_size = 4096;
        //len = mp3_decode_frame(c, outbuf, &out_size, inFile, inFileRemaining);
        len = mp3_decode_frame(c, p, &out_size, inFile, inFileRemaining);
        if (len < 0) {
            write(2, "Error while decoding\n", 21);
            return -1;
        }
        if (out_size > 0) {
	        //if(!pr) { fprintf(stderr, "decoded frame size = %i\n", out_size); pr = 1; }
            /* if a frame has been decoded, output it */
            int16_t tmpbuf[16384];
            unsigned int k;
            c->frame_number++;
            for (k = 0; k < out_size; k++) {
                tmpbuf[2*k  ] = WORD2INT(32768.0f*p[0][k]);
                tmpbuf[2*k+1] = WORD2INT(32768.0f*p[c->nb_channels - 1][k]);
            }
            //conv_fltp_to_flt_2ch_sse(tmpbuf, p, (out_size>>1));
            if (c->nb_channels == 1) {
                out_size <<= 1;
            }
            write(out_fd, tmpbuf, out_size*sizeof(int16_t));
        }
        inFile += len;
    }

    close(out_fd);
    return 0;
}

